package net.blumia.pcmdroid.databases

import android.content.Context
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import net.blumia.pcmdroid.model.Server
import net.blumia.pcmdroid.model.ServerDao

// Annotates class to be a Room Database with a table (entity) of the Word class
@Database(entities = [Server::class], version = 1, exportSchema = false)
public abstract class PcmRoomDatabase : RoomDatabase() {
    abstract fun serverDao(): ServerDao

    companion object {
        // Singleton prevents multiple instances of database opening at the same time.
        @Volatile
        private var INSTANCE: PcmRoomDatabase? = null

        fun getDatabase(context: Context): PcmRoomDatabase {
            // if the INSTANCE is not null, then return it,
            // if it is, then create the database
            return INSTANCE ?: synchronized(this) {
                val instance = Room.databaseBuilder(
                    context.applicationContext,
                    PcmRoomDatabase::class.java,
                    "pcm_database"
                ).build()
                INSTANCE = instance
                // return instance
                instance
            }
        }
    }
}