package net.blumia.pcmdroid.model

import android.util.Log
import androidx.lifecycle.LiveData
import androidx.room.*
import kotlinx.coroutines.flow.Flow
import org.json.JSONException
import org.json.JSONObject

@Entity(tableName = "server_table")
data class Server (
    @PrimaryKey(autoGenerate = true)
    @ColumnInfo(name = "serverId")
    val serverId: Long? = null,
    @ColumnInfo(name = "url")
    val url: String,
    @ColumnInfo(name = "name")
    val name: String,
    @ColumnInfo(name = "shortName")
    val shortName: String,
    @ColumnInfo(name = "mediaBaseUrl")
    val mediaBaseUrl: String,
    @ColumnInfo(name = "preferredFormats", defaultValue = "")
    val preferredFormats: String,
    @ColumnInfo(name = "baseFolderName")
    val baseFolderName: String,
) {
    fun displayName(): String {
        return name
    }

    fun shortDisplayName(): String {
        return if (shortName.isNotEmpty()) shortName else name
    }
}

@Dao
interface ServerDao {

    @Query("SELECT * FROM server_table") //  ORDER BY wtf ASC
    fun getServers(): Flow<List<Server>>

    @Query("SELECT * FROM server_table WHERE serverId=:id")
    fun getServerById(id: Long): Flow<Server>

    @Insert(onConflict = OnConflictStrategy.IGNORE)
    suspend fun insert(server: Server)

    @Update
    suspend fun update(server: Server)

    @Delete
    suspend fun delete(server: Server)

    @Query("DELETE FROM server_table")
    suspend fun deleteAll()
}

fun parseServerFromJsonString(jsonString: String): Map<String, String> {

    val kv = mutableMapOf<String, String>()

    try {
        val jsonObj = JSONObject(jsonString)
        val result = jsonObj.getJSONObject("result")

        kv["serverName"] = result.getString("serverName")
        kv["serverShortName"] = result.getString("serverShortName")
        kv["baseFolderNameHint"] = result.getString("baseFolderNameHint")
        kv["preferredFormatsHint"] = result.getString("preferredFormatsHint")
        kv["mediaRootUrl"] = result.getString("mediaRootUrl")
    } catch (e: JSONException) {
        e.printStackTrace()
        Log.d("vvv", jsonString)
    }

    return kv;
}