package net.blumia.pcmdroid.repository

import androidx.annotation.WorkerThread
import kotlinx.coroutines.flow.Flow
import net.blumia.pcmdroid.model.Server
import net.blumia.pcmdroid.model.ServerDao

// Declares the DAO as a private property in the constructor. Pass in the DAO
// instead of the whole database, because you only need access to the DAO
class ServerRepository(private val serverDao: ServerDao) {

    // Room executes all queries on a separate thread.
    // Observed Flow will notify the observer when the data has changed.
    val allServers: Flow<List<Server>> = serverDao.getServers()

    // By default Room runs suspend queries off the main thread, therefore, we don't need to
    // implement anything else to ensure we're not doing long running database work
    // off the main thread.
    @Suppress("RedundantSuspendModifier")
    @WorkerThread
    suspend fun insert(server: Server) {
        serverDao.insert(server)
    }

    @Suppress("RedundantSuspendModifier")
    @WorkerThread
    suspend fun update(server: Server) {
        serverDao.update(server)
    }

    @Suppress("RedundantSuspendModifier")
    @WorkerThread
    suspend fun delete(server: Server) {
        serverDao.delete(server)
    }

    @Suppress("RedundantSuspendModifier")
    @WorkerThread
    fun serverById(id: Long): Flow<Server> {
        return serverDao.getServerById(id)
    }
}