package net.blumia.pcmdroid.ui.screen.main

import android.net.Uri
import androidx.compose.foundation.clickable
import androidx.compose.foundation.horizontalScroll
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.material.Icon
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.NavigateNext
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import net.blumia.pcmdroid.model.Folder

@Composable
fun Breadcrumb(
    folder: Folder? = null,
    onFolderClicked: (Folder) -> Unit = {}
) {
    val iconLeftPadding = 16.dp
    val iconVerticalPadding = 12.dp
    Row(
        modifier = Modifier
            .fillMaxWidth()
            .horizontalScroll(rememberScrollState())
            .padding(
                horizontal = iconLeftPadding,
                vertical = iconVerticalPadding,
            ),
        verticalAlignment = Alignment.CenterVertically,
    ) {
        val folderPath = folder?.folderPath ?: ""
        val pathSeg = Uri.parse(folderPath).pathSegments

        pathSeg.forEachIndexed { idx, seg ->
            val isLastSeg = seg == pathSeg.last()

            Text(
                modifier = Modifier.clickable {
                    val segList = pathSeg.take(idx + 1)
                    val clickedFolder = Folder(
                        folderPath = segList.joinToString("/")
                    )

                    onFolderClicked(clickedFolder)
                },
                text = seg,
                style = MaterialTheme.typography.body1,
                color = if (isLastSeg) MaterialTheme.colors.primary else MaterialTheme.colors.onSurface
            )

            if (!isLastSeg) {
                Icon(
                    modifier = Modifier.padding(horizontal = 5.dp),
                    imageVector = Icons.Filled.NavigateNext,
                    contentDescription = null,
                )
            }
        }
    }
}

@Preview(showBackground = true)
@Composable
fun BreadcrumbPreview() {
    Breadcrumb(Folder("""Deep/Dark/Folder"""))
}