package net.blumia.pcmdroid.ui.screen.main

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.*
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.AddCircle
import androidx.compose.material.icons.filled.LibraryMusic
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material.icons.filled.Settings
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import net.blumia.pcmdroid.R
import net.blumia.pcmdroid.model.Folder
import net.blumia.pcmdroid.model.Server

@Preview(showBackground = true)
@OptIn(ExperimentalMaterialApi::class)
@Composable
fun SidebarContent(
    currentServer: Server? = null,
    servers: List<Server> = listOf(),
    drawerFolders: List<Folder> = listOf(),
    addServerActionTriggered: () -> Unit = {},
    onServerItemIconClicked: (Server) -> Unit = {},
    onServerMenuIconClicked: () -> Unit = {},
    onDrawerFolderItemClicked: (Folder) -> Unit = {},
    settingsActionTriggered: () -> Unit = {},
) {
    val currentServerApiUrl = currentServer?.url

    Row(modifier = Modifier.fillMaxSize()) {
        NavigationRail {
            servers.forEachIndexed { index, server ->
                NavigationRailItem(
                    icon = {
                        Icon(
                            Icons.Filled.LibraryMusic, server.name
                        )
                    },
                    label = {
                        Text(server.shortDisplayName())
                    },
                    selected = server.url == currentServerApiUrl,
                    onClick = {
                        onServerItemIconClicked(server)
                    }
                )
            }

            NavigationRailItem(
                icon = { Icon(Icons.Filled.AddCircle, null) },
                label = { Text(stringResource(id = R.string.add)) },
                selected = false,
                onClick = addServerActionTriggered,
            )

            Spacer(modifier = Modifier.weight(1f))

            NavigationRailItem(
                icon = { Icon(Icons.Filled.Settings, null) },
                label = { Text(stringResource(id = R.string.settings)) },
                selected = false,
                onClick = settingsActionTriggered,
            )
        }

        Column(
            modifier = Modifier
                .weight(1f)
        ) {
            ListItem(
                text = {
                    Text(
                        text = currentServer?.displayName() ?: "No server selected",
                        style = MaterialTheme.typography.subtitle1
                    )
                },
                trailing = {
                    IconButton(onClick = onServerMenuIconClicked) {
                        Icon(imageVector = Icons.Filled.MoreVert, contentDescription = "Menu")
                    }
                }
            )

            Divider()

            Column(
                modifier = Modifier
                    .weight(1f, fill = true)
                    .verticalScroll(rememberScrollState())
            ) {
                drawerFolders.forEach { folder ->
                    ListItem(
                        modifier = Modifier.clickable{
                            onDrawerFolderItemClicked(folder)
                        },
                        text = {
                            Text(
                                text = folder.displayName(),
                                style = MaterialTheme.typography.subtitle1
                            )
                        }
                    )
                }
            }
        }
    }
}