package net.blumia.pcmdroid.viewmodel

import android.util.Log
import android.webkit.URLUtil
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import net.blumia.pcmdroid.model.Server
import net.blumia.pcmdroid.model.parseFromJsonString
import net.blumia.pcmdroid.model.parseServerFromJsonString
import okhttp3.FormBody
import okhttp3.OkHttpClient
import okhttp3.Request

class AddServerViewModel : ViewModel() {
//    private val httpClient: OkHttpClient = OkHttpClient()

    private val _apiUrl: MutableLiveData<String> = MutableLiveData("")
    val apiUrl: LiveData<String> = _apiUrl
    fun setApiUrl(url: String) {
        // FIXME: do we really need to use postValue here?
        //        will the setter for the LiveData object trigger observer?
        _apiUrl.postValue(url)
    }

    private val _nameStr: MutableLiveData<String> = MutableLiveData("")
    val nameStr: LiveData<String> = _nameStr
    fun setNameStr(name: String) {
        _nameStr.postValue(name)
    }

    private val _shortNameStr: MutableLiveData<String> = MutableLiveData("")
    val shortNameStr: LiveData<String> = _shortNameStr
    fun setShortNameStr(shortName: String) {
        _shortNameStr.postValue(shortName)
    }

    private val _baseFolderNameStr: MutableLiveData<String> = MutableLiveData("")
    val baseFolderNameStr: LiveData<String> = _baseFolderNameStr
    fun setBaseFolderNameStr(str: String) {
        _baseFolderNameStr.postValue(str)
    }

    private val _mediaBaseUrlStr: MutableLiveData<String> = MutableLiveData("")
    val mediaBaseUrlStr: LiveData<String> = _mediaBaseUrlStr
    fun setMediaBaseUrlStr(str: String) {
        _mediaBaseUrlStr.postValue(str)
    }

    private val _preferredFormatsStr: MutableLiveData<String> = MutableLiveData("")
    val preferredFormatsStr: LiveData<String> = _preferredFormatsStr
    fun setPreferredFormatsStr(str: String) {
        _preferredFormatsStr.postValue(str)
    }

    fun isApiUrlValid(): Boolean {
        return URLUtil.isNetworkUrl(apiUrl.value)
    }

    fun fetchContentFromApiUrl() {
        val httpClient: OkHttpClient = OkHttpClient()

        viewModelScope.launch(context = Dispatchers.IO) {

            val formBody = FormBody.Builder()
                .add("do", "getserverinfo")
                .build()

            val request = Request.Builder()
//                .url("http://localhost/")
                .url(apiUrl.value!!)
                .post(formBody)
                .build()

            try {
                httpClient.newCall(request).execute().use { response ->
                    if (!response.isSuccessful) {
                        // TODO: non-200 response will go to here too.
                        return@use
                    }
                    val kv = parseServerFromJsonString(response.body!!.string())
                    Log.d("vvv", kv.toString())
                    if (kv["serverName"] != null) setNameStr(kv["serverName"]!!)
                    if (kv["serverShortName"] != null) setShortNameStr(kv["serverShortName"]!!)
                    if (kv["baseFolderNameHint"] != null) setBaseFolderNameStr(kv["baseFolderNameHint"]!!)
                    if (kv["preferredFormatsHint"] != null) setPreferredFormatsStr(kv["preferredFormatsHint"]!!)
                    if (kv["mediaRootUrl"] != null) setMediaBaseUrlStr(kv["mediaRootUrl"]!!)
                }
            } catch (e: java.io.IOException) {
                e.printStackTrace()
            }
        }
    }
}